# -*- coding: utf-8 -*-
# This is here for Python 2.7 support.

# ¯\_(ツ)_/¯
"""
A person shrugging their shoulders to indicate a lack of knowledge about a 
particular topic, or a lack of care about the result of a situation.
"""
